/*!
 * @vueblocks/global v1.0.0-alpha.5
 * (c) 2021 xiaoluoboding
 * @license MIT
 */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var vue = require('vue');
var core = require('echarts/core');
var resizeObserver = require('@juggle/resize-observer');
var charts = require('echarts/charts');
var cloneDeep = require('lodash.clonedeep');
var components = require('echarts/components');
var renderers = require('echarts/renderers');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var cloneDeep__default = /*#__PURE__*/_interopDefaultLegacy(cloneDeep);

/* eslint-disable @typescript-eslint/no-explicit-any */
const toKebabCase = (str) => str
    .replace(/([A-Z])([A-Z])/g, '$1-$2')
    .replace(/([a-z])([A-Z])/g, '$1-$2')
    .replace(/[\s_]+/g, '-')
    .toLowerCase();
const toCamelCase = (arr) => arr
    .map((str, idx) => idx > 0 ? str.charAt(0).toLocaleUpperCase() + str.slice(1) : str)
    .join('');
const isEmpty = (prop) => prop === null || prop === undefined;
// Copied from
// https://github.com/vuejs/vue-next/blob/5a7a1b8293822219283d6e267496bec02234b0bc/packages/shared/src/index.ts#L40-L41
const isOn = (key) => /^on[^a-z]/.test(key);
const omitOn = (attrs) => {
    const result = {};
    for (const key in attrs) {
        if (!isOn(key)) {
            result[key] = attrs[key];
        }
    }
    return result;
};
const withInstall = (comp) => {
    const c = comp;
    c.install = (app) => {
        app.component(c.name, c);
    };
    return c;
};
const unrefElement = (refEl) => {
    return vue.unref(refEl);
};

/* eslint-disable @typescript-eslint/no-explicit-any */
const useAttrs = (attrs) => {
    const echartsAttrs = vue.computed(() => omitOn(attrs));
    const echartsEvents = {};
    Object.keys(attrs)
        .filter(key => isOn(key) && typeof attrs[key] === 'function')
        .forEach(key => {
        const [, ...rest] = toKebabCase(key).split('-');
        const eventKey = rest.length === 1
            ? rest.join('')
            : toCamelCase(rest);
        echartsEvents[eventKey] = attrs[key];
    });
    return {
        echartsAttrs,
        echartsEvents
    };
};

/**
 * It immediately detects when an element resizes and provides accurate sizing information back to the handler.
 * @param target
 * @param callback
 * @param options
 */
const useResizeObserver = (target, callback, options = {}) => {
    let ro;
    const isSupported = window && 'ResizeObserver' in window;
    const stopWatch = vue.watch(() => unrefElement(target), (el) => {
        if (isSupported && window && el) {
            ro = new resizeObserver.ResizeObserver(callback);
            ro.observe(el, options);
        }
    }, { immediate: true });
    const stop = () => {
        ro && ro.disconnect();
        ro = undefined;
        stopWatch();
    };
    vue.onUnmounted(stop);
    return {
        stop
    };
};

/* eslint-disable @typescript-eslint/no-explicit-any */
var Chart$9 = vue.defineComponent({
    name: 'VeChart',
    inheritAttrs: false,
    props: {
        option: Object,
        initOptions: Object,
        theme: [String, Object],
        setOptionOpts: Object,
        height: { type: Number, default: 400 },
        needUpdate: { type: Boolean, default: false }
    },
    setup(props, { attrs }) {
        const option = vue.toRef(props, 'option');
        const theme = vue.toRef(props, 'theme');
        const initOptions = vue.toRef(props, 'initOptions');
        const setOptionOpts = vue.toRef(props, 'setOptionOpts');
        const needUpdate = vue.toRef(props, 'needUpdate');
        const echartsRef = vue.ref();
        const echartsInstance = vue.shallowRef();
        const canvasRect = vue.ref({});
        const { echartsEvents } = useAttrs(attrs);
        // console.log(option.value)
        const echartsStyle = {
            width: 'auto',
            height: `${props.height}px`
        };
        const resize = () => {
            if (echartsInstance.value) {
                echartsInstance.value.resize();
            }
        };
        const dispose = () => {
            if (echartsInstance.value) {
                echartsInstance.value.dispose();
                echartsInstance.value = undefined;
            }
        };
        const delegateEvents = (instance) => {
            Object.keys(echartsEvents).forEach(key => {
                const handler = echartsEvents[key];
                if (!handler)
                    return;
                if (key.indexOf('zr:') === 0) {
                    instance.getZr().on(key.slice(3).toLowerCase(), handler);
                }
                else {
                    instance.on(key, handler);
                }
            });
        };
        const init = (option) => {
            if (!echartsRef.value)
                return;
            echartsInstance.value = core.init(echartsRef.value, props.theme, {
                renderer: 'canvas'
            });
            // console.log(echartsInstance.value)
            option && echartsInstance.value.setOption(option);
            delegateEvents(echartsInstance.value);
            vue.nextTick(resize);
        };
        const setOption = (option, opts) => {
            if (!echartsInstance.value) {
                init(option);
            }
            else {
                echartsInstance.value.setOption(option, opts);
            }
        };
        useResizeObserver(echartsRef, ([entry]) => {
            canvasRect.value = entry.contentRect;
            resize();
        });
        vue.watch(needUpdate, (needUpdate) => {
            if (needUpdate && option.value) {
                if (setOptionOpts.value) {
                    setOption(option.value, setOptionOpts.value);
                }
                else {
                    setOption(option.value);
                }
            }
        }, { deep: true });
        vue.watch([theme, initOptions], () => {
            dispose();
            init();
        }, { deep: true });
        vue.onMounted(() => {
            option.value && init(option.value);
        });
        vue.onUnmounted(dispose);
        return () => vue.h('div', {
            ref: echartsRef,
            style: echartsStyle
        });
    }
});

const VeChart = withInstall(Chart$9);

core.use([
    components.GridComponent,
    renderers.CanvasRenderer
]);

/* eslint-disable @typescript-eslint/no-explicit-any */
var HocChart = vue.defineComponent({
    props: {
        /**
         * echarts options
         * ve-charts delegate all echarts options as component props
         * https://echarts.apache.org/zh/option.html
         */
        title: Object,
        legend: Object,
        grid: Object,
        xAxis: [Object, Array],
        yAxis: [Object, Array],
        polar: Object,
        radiusAxis: Object,
        angleAxis: Object,
        radar: [Object, Array],
        dataZoom: [Object, Array],
        visualMap: [Object, Array],
        tooltip: Object,
        axisPointer: Object,
        toolbox: Object,
        brush: Object,
        geo: Object,
        parallel: Object,
        parallelAxis: Array,
        singleAxis: Array,
        timeline: Object,
        graphic: Object,
        calendar: Object,
        dataset: Object,
        aria: Object,
        series: [Object, Array],
        darkMode: { type: [Boolean, String], default: 'auto' },
        color: Array,
        backgroundColor: [Object, String],
        textStyle: Object,
        animation: { type: Boolean, default: true },
        animationThreshold: Number,
        animationDuration: [Number, Function],
        animationEasing: [String, Function],
        animationDelay: [Number, Function],
        animationDurationUpdate: [Number, Function],
        animationEasingUpdate: [String, Function],
        animationDelayUpdate: [String, Function],
        stateAnimation: Object,
        blendMode: String,
        hoverLayerThreshold: Number,
        useUTC: { type: Boolean, default: false },
        options: Object,
        media: Array,
        // ve-charts basic props
        data: [Object, Array],
        settings: [Object, Array],
        loading: { type: Boolean, default: false },
        emptyText: String,
        // ve-charts common props
        option: {
            type: [Object, Array],
            default: () => { return {}; }
        }
    },
    data: () => ({
        initOptions: {
            renderer: 'canvas'
        },
        needUpdate: false,
        setOptionOpts: {},
        mergedOption: {}
    }),
    computed: {
        chartOpts() {
            return {
                option: this.mergedOption,
                initOptions: this.initOptions,
                needUpdate: this.needUpdate,
                setOptionOpts: this.setOptionOpts
            };
        }
    },
    created() {
        this.mergedOption = this.option;
        Object.keys(this.$props)
            // .forEach((prop: string) => (this.$props[prop] === undefined) && delete this.$props[prop])
            .filter((prop) => !/data|settings|height|loading|emptyText/.test(prop))
            .forEach((prop) => {
            this.$watch(prop, (val) => {
                if (!isEmpty(val)) {
                    // merge echarts default option
                    const option = prop === 'option'
                        ? this.option
                        : { [prop]: val };
                    this.mergePropsToOption(option);
                }
                this.forcedUpdate();
            }, {
                // TODO only object types need deep watch
                deep: true,
                immediate: true
            });
        });
    },
    beforeUnmount() {
        this.mergedOption = {};
    },
    methods: {
        mergePropsToOption(props) {
            const option = cloneDeep__default['default'](this.mergedOption);
            this.mergedOption = { ...option, ...props };
        },
        forcedUpdate() {
            // manual update echarts options
            this.needUpdate = true;
            this.$nextTick(() => { this.needUpdate = false; });
        }
    },
    render() {
        return vue.h(VeChart, this.chartOpts);
    }
});

core.use([charts.BarChart]);
var Chart$8 = vue.defineComponent({
    name: 'BarChart',
    extends: HocChart,
    setup(props) {
        console.log(props);
    }
});

const BarChart = withInstall(Chart$8);

core.use([charts.LineChart]);
var Chart$7 = vue.defineComponent({
    name: 'LineChart',
    extends: HocChart,
    setup(props) {
        console.log(props);
    }
});

const LineChart = withInstall(Chart$7);

core.use([charts.PieChart]);
var Chart$6 = vue.defineComponent({
    name: 'PieChart',
    extends: HocChart,
    setup(props) {
        console.log(props);
    }
});

const PieChart = withInstall(Chart$6);

core.use([charts.FunnelChart]);
var Chart$5 = vue.defineComponent({
    name: 'FunnelChart',
    extends: HocChart,
    setup(props) {
        console.log(props);
    }
});

const FunnelChart = withInstall(Chart$5);

core.use([charts.GaugeChart]);
var Chart$4 = vue.defineComponent({
    name: 'GaugeChart',
    extends: HocChart,
    setup(props) {
        console.log(props);
    }
});

const GaugeChart = withInstall(Chart$4);

core.use([charts.RadarChart]);
var Chart$3 = vue.defineComponent({
    name: 'RadarChart',
    extends: HocChart,
    setup(props) {
        console.log(props);
    }
});

const RadarChart = withInstall(Chart$3);

core.use([charts.ScatterChart]);
var Chart$2 = vue.defineComponent({
    name: 'ScatterChart',
    extends: HocChart,
    setup(props) {
        console.log(props);
    }
});

const ScatterChart = withInstall(Chart$2);

core.use([charts.TreeChart]);
var Chart$1 = vue.defineComponent({
    name: 'TreeChart',
    extends: HocChart,
    setup(props) {
        console.log(props);
    }
});

const TreeChart = withInstall(Chart$1);

core.use([charts.TreemapChart]);
var Chart = vue.defineComponent({
    name: 'TreemapChart',
    extends: HocChart,
    setup(props) {
        console.log(props);
    }
});

const TreemapChart = withInstall(Chart);

const componentList = [
    VeChart,
    BarChart,
    LineChart,
    PieChart,
    FunnelChart,
    GaugeChart,
    RadarChart,
    ScatterChart,
    TreeChart,
    TreemapChart
];
const install = (app) => {
    componentList.forEach(component => {
        app.component(component.name, component);
    });
};

exports.BarChart = BarChart;
exports.FunnelChart = FunnelChart;
exports.GaugeChart = GaugeChart;
exports.LineChart = LineChart;
exports.PieChart = PieChart;
exports.RadarChart = RadarChart;
exports.ScatterChart = ScatterChart;
exports.TreeChart = TreeChart;
exports.TreemapChart = TreemapChart;
exports.VeChart = VeChart;
exports.default = install;
